require "time"

module Testcube
  class BuildEnv
    def self.current
      # TODO could also get some of this data from github api
      {
        branch: branch,
        pr: pr,
        build_url: build_url,
        head: parse_from_git('HEAD'),
        merge_base: parse_from_git(merge_base)
      }
    end

    # TODO: Add support for other CI systems
    def self.branch
      ENV['BUILDKITE_BRANCH'] || git('rev-parse --abbrev-ref HEAD')
    end

    def self.pr
      ENV['BUILDKITE_PULL_REQUEST']
    end

    def self.build_url
      ENV['BUILDKITE_BUILD_URL']
    end

    def self.parse_from_git(sha)
      out = git("show #{sha} --no-patch")
      return {} if !out

      # TODO: dont rescue nil
      return {
        sha: (out.match(/commit ([0-9a-f]+)\n/)[1] rescue nil),
        author: (out.match(/Author: (.*?) <(.*)>\n/)[1] rescue nil),
        ts: (Time.parse(out.match(/Date: (.*?)\n/)[1].strip).to_i rescue nil),
        author_email: (out.match(/Author: (.*?) <(.*)>\n/)[2] rescue nil),
        message: (out.lines[3..(out.lines.length)].find { |l| l.strip.length > 0 }.strip[0..50] rescue nil)
      }
    end

    def self.merge_base
      git("merge-base HEAD master")
    end

    def self.git(cmd)
      `git #{cmd}`.strip
    rescue Errno::ENOENT
      nil
    end
  end
end
