require 'singleton'

module Testcube
  class Tracker
    include Singleton

    attr_reader :test_files_with_time
    attr_writer :current_test_path, :current_line_number, :current_exception, :current_description

    def initialize
      set_defaults
    end

    def start_timer
      @start_time = now_without_mock_time.to_f
    end

    def stop_timer
      execution_time = @start_time ? now_without_mock_time.to_f - @start_time : 0.0
      @test_files_with_time.push({
        test_file: current_test_path,
        line_number: current_line_number,
        description: current_description,
        status: current_exception ? :fail : :pass,
        # TODO: we could have a better way to filter the backtrace
        backtrace: current_exception ? current_exception.backtrace.select { |s| s !~ /gems\/.*(rspec|bin)/ } : nil,
        execution_time: execution_time
      })
    end

    def current_test_path
      raise("current_test_path needs to be set") unless @current_test_path
      @current_test_path.sub(/^\.\//, '')
    end

    def current_line_number
      raise("current_line_number needs to be set") unless @current_line_number
      @current_line_number
    end

    def current_description
      raise("current_description needs to be set") unless @current_description
      @current_description
    end

    def current_exception
      # can be nil
      @current_exception
    end

    def total_time
      test_files_with_time.map { |res| res[:execution_time] }.sum
    end

    def num_specs
      test_files_with_time.length
    end

    private

    def set_defaults
      @global_time = 0
      @test_files_with_time = []
    end

    def now_without_mock_time
      if defined?(Timecop)
        Time.now_without_mock_time
      else
        Time.now
      end
    end
  end
end
